/**
 * AmbientTalk/2 Project
 * (c) Programming Technology Lab, 2006 - 2010
 * Authors: Software Languages Lab - Ambient Group
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package edu.vub.at.android.interpreter;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import mobileAgent.GeneralMobileAgent;
import residenceAgent.ResidenceAgent;
import android.app.Activity;
import android.app.TabActivity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.Editable;
import android.text.SpannableString;
import android.text.TextWatcher;
import android.text.style.ForegroundColorSpan;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ScrollView;
import android.widget.TabHost;
import android.widget.TabHost.OnTabChangeListener;
import android.widget.TextView;
import android.widget.Toast;
import atVM.AFrame;
import atVM.ApplicationExistsException;
import atVM.IApplication;
import atVM.TestInit;
import edu.vub.at.android.util.Constants;
import edu.vub.at.android.util.IATSettings;
import edu.vub.at.android.util.IntConstants;
import edu.vub.at.android.util.OutputAT;
import edu.vub.at.eval.Evaluator;

/**
 * This Activity is launched after the Greeter view. It consist of: - Edit and
 * Evaluate tab. - The graphics elements of the two tabs are kept jointly here.
 * - a Handler to receive the input/output/err messages of the AT interpreter
 * from OutputAT. - Menu Options (@see onCreateOptionsMenu).
 */
public class ATAndroid extends TabActivity {
	// iatOptions contains the information about iat command line option
	// which are set in the Setting view
	private IATSettings.IATOptions iatOptions_;
	public static final int SIM_REQUEST = 100;

	// Editor tab consists of an inputTex and a button to eval.
	// This button allows to switch between Evaluator and Editor
	// tab without triggering the evaluation of the inputtext.
	private EditText inputText_;
	private Button evalButton_;
	private Button testButton_;
	private Button initButton_;
	private static AFrame aFrame1;
	private boolean isSimEnabled = false;
	private boolean randomRates = true;
	private double simDuration = 60000;
	private double onlineRate;
	private double offlineRate;
	private double delay = 2000;
	// stores the file which is currently being displayed in the Editor tab.
	// each time the user evaluates the code, the file is first saved.
	// If the user tried to evaluate a new file without explicitly saving it,
	// the application saves a tmp file for the current contents of the Editor
	// tab.
	private File fileToEval_ = null;

	// Evaluator tab consist of an scrollView which contains
	// the output text view and an input text used to send small
	// commands to the interpreter without having to write it
	// in the file currently being edited.
	private TextView outputText_;
	private static ScrollView scrollView_;
	private EditText inputSnippet_;

	// this handle is used to display the output of the evaluation
	// in the Evaluator tab. OutputAT sends a message to this handle
	// for each error/output message to be displayed.
	public static Handler myATViewUpdateHandler_ = null;
	// used by threads sending commands to iat
	private final Activity act_ = this;

	/**
	 * Called when the activity is first created.
	 * 
	 * If the user selected to edit a recently used file, the intent has one
	 * extra object named IntConstants.INTENT_LOAD_FILE which contains the
	 * fileName of a recently used file to browse.
	 * */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		// Code to setup the TABS
		Resources res = getResources(); // Resource object to get Drawables
		final TabHost tabHost = getTabHost(); // The activity TabHost
		TabHost.TabSpec spec; // Reusable TabSpec for each tab

		// Initialize a TabSpec for each tab and add it to the TabHost
		spec = tabHost
				.newTabSpec(getString(R.string.edit))
				.setIndicator(getString(R.string.edit),
						res.getDrawable(R.drawable.ic_tab_editor))
				.setContent(R.id.editor_tab);
		tabHost.addTab(spec);
		// Do the same for the other tabs
		spec = tabHost
				.newTabSpec(getString(R.string.eval))
				.setIndicator(getString(R.string.eval),
						res.getDrawable(R.drawable.ic_tab_eval))
				.setContent(R.id.eval_tab);
		tabHost.addTab(spec);
		tabHost.setCurrentTab(0);

		tabHost.setOnTabChangedListener(new OnTabChangeListener() {
			public void onTabChanged(String tabId) {
				// When the user switches to the Edit tab, the contents of the
				// file that was evaluated
				// get loaded again in the editor.
				if ((getString(R.string.edit).equals(tabId))) {
					// display the file again.
					loadContentOfFileInEditTab(fileToEval_);
				}
				// When the user switches to the Eval tab, the contents of the
				// editor get saved.
				// TODO: Is this too heavy? should we only save the contents of
				// the file when eval is clicked?
				if ((getString(R.string.eval).equals(tabId))) {
					saveContentOfInputTextInEvalTab();
				}
			}
		});

		final Activity me = this;

		// Code to create the ui elements inside the edit tab
		inputText_ = (EditText) this.findViewById(R.id.editorText);
		evalButton_ = (Button) this.findViewById(R.id.evalButton);
		evalButton_.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				// when the users clicks on the eval button,
				// the contents of the editor get saved before evaluating them.
				saveContentOfInputTextInEvalTab();
				getTabHost().setCurrentTab(1);
				IATManager.evaluate(inputText_.getText().toString(), me);
			}
		});

		// Code to create the ui elements inside the eval tab
		scrollView_ = (ScrollView) this.findViewById(R.id.scrollviewEval);
		outputText_ = (TextView) this.findViewById(R.id.outputEval);

		inputSnippet_ = (EditText) this.findViewById(R.id.inputSnippet);
		// the content of the inputSnippet gets evaluated when the user
		// writes Enter on it.

		inputSnippet_.addTextChangedListener(new TextWatcher() {

			public void afterTextChanged(Editable arg0) {
				// no-op
			}

			public void beforeTextChanged(CharSequence s, int start, int count,
					int after) {
				// no-op
			}

			public void onTextChanged(CharSequence s, int start, int before,
					int count) {
				// his method is called to notify you that, within s,
				// the count characters beginning at start have just
				// replaced old text that had length before.
				for (int i = start; i < start + count; i++) {
					if (s.charAt(i) == '\n') {
						String code = inputSnippet_.getText().toString();
						inputSnippet_.setText("");
						if (!code.equals("")) {
							OutputAT.in.println(code);
							IATManager.evaluate(code.trim(), me);
						}
					}
				}
			}

		});

		/*
		 * // Does not work on all phones :( inputSnippet.setOnKeyListener(new
		 * OnKeyListener() { public boolean onKey(View v, int keyCode, KeyEvent
		 * event) { // If the event is a key-down event on the "enter" button if
		 * (keyCode == KeyEvent.KEYCODE_ENTER) { // Perform action on key press
		 * String code = inputSnippet.getText().toString();
		 * inputSnippet.setText(""); if (!code.equals("")) {
		 * OutputAT.in.println(code); evaluate(code); } return true; } return
		 * false; } });
		 */

		// If this method is called by a screen rotation, reuse the previous
		// text.
		Object savedText = getLastNonConfigurationInstance();
		if (savedText != null) {
			outputText_.setText((CharSequence) savedText);
			scrollView_.fullScroll(View.FOCUS_DOWN);
		}

		// setup the view update handler and attach the output to it.
		myATViewUpdateHandler_ = new ATViewUpdateHandler();
		OutputAT.in.attach(myATViewUpdateHandler_);
		OutputAT.out.attach(myATViewUpdateHandler_);
		OutputAT.err.attach(myATViewUpdateHandler_);
		// get the latest iat options
		iatOptions_ = IATSettings.getIATOptions(this);
		// This class may be called with an extra object in its intent
		// if the user selected a recently edited file in Greeter.java
		if (getIntent().hasExtra(IntConstants._INTENT_LOAD_FILE_)) {
			String path = getIntent().getStringExtra(
					IntConstants._INTENT_LOAD_FILE_);
			fileToEval_ = new File(path);
			loadContentOfFileInEditTab(fileToEval_);
		}

		testButton_ = (Button) this.findViewById(R.id.testButton);
		class TestButtonClickListener implements OnClickListener {
			public TabActivity getTabActivity() {
				return tabActivity;
			}

			public void setTabActivity(TabActivity tabActivity) {
				this.tabActivity = tabActivity;
			}

			private TabActivity tabActivity;

			public void onClick(View v) {
				ATAndroid atAndroid = (ATAndroid) this.getTabActivity();
				atAndroid.getTabHost().setCurrentTab(1);
				IATManager.evaluate("\n", me);
				class MyThread extends Thread implements IApplication {
					public void run() {
						if (aFrame1 == null) {
							System.out.println("initializing aFrame");
							aFrame1 = new AFrame(IATManager._IAT_);
						}
						ResidenceAgent ra1 = aFrame1.getResidenceAgent();
						System.out.println("Residence Agent Initialized: \""
								+ ra1.getResidenceAgentName() + "\"");
						String appName = "test";
						System.out.println("ra1.register() result: ");
						try {
							ra1.register(appName, new TestInit());
						} catch (ApplicationExistsException e) {
							e.printStackTrace();
						}
						System.out.println("current system time: \""
								+ ra1.getCurrentTime() + "\"");
						GeneralMobileAgent ma1 = aFrame1.createMobileAgent();
						System.out.println("mobile agent created: "
								+ ma1.getId());
						ma1.getApplicationSpecificMobileAgent("ipAddr", "1");
						ma1.setOwner(ra1.getResidenceAgentName());
						ma1.setAppName(appName);
						ma1.setTransitionMode("allNodes");
						ma1.setDelay((int) delay);
						Collection<String> set = new ArrayList<String>();
						ma1.setTransitionSet(set);
						ma1.execute();
						ma1.transitToNextNode(true, true);
					}

					public void mobileAgentComeBack(String id, String data) {
						String[] array = data.split("#takenAwayDataSplit#");
						String text = "mobile agent executing result: \n";
						for (int i = 0; i < array.length; i++) {
							String line = array[i];
							text = text + line + "\n";
						}
						System.out.println("return result: " + text);
					}
				}
				MyThread t = new MyThread();
				t.start();
			}
		}
		TestButtonClickListener testButtonClickListener = new TestButtonClickListener();
		testButtonClickListener.setTabActivity(this);
		testButton_.setOnClickListener(testButtonClickListener);
		initButton_ = (Button) this.findViewById(R.id.initButton);
		class InitButtonClickListener implements OnClickListener {
			public TabActivity getTabActivity() {
				return tabActivity;
			}

			public void setTabActivity(TabActivity tabActivity) {
				this.tabActivity = tabActivity;
			}

			private TabActivity tabActivity;

			public void onClick(View v) {
				ATAndroid atAndroid = (ATAndroid) this.getTabActivity();
				atAndroid.getTabHost().setCurrentTab(1);
				IATManager.evaluate("\n", me);
				class MyThread extends Thread {
					public void run() {
						if (aFrame1 == null) {
							System.out.println("initializing aFrame");
							aFrame1 = new AFrame(IATManager._IAT_);
						}
						ResidenceAgent ra1 = aFrame1.getResidenceAgent();
						System.out.println("Residence Agent Initialized: \""
								+ ra1.getResidenceAgentName() + "\"");
						tabHost.setCurrentTab(1);
					}
				}
				MyThread t = new MyThread();
				t.start();
			}
		}
		InitButtonClickListener initButtonClickListener = new InitButtonClickListener();
		initButtonClickListener.setTabActivity(this);
		initButton_.setOnClickListener(initButtonClickListener);
		if (randomRates) {
			offlineRate = 0.1;
			onlineRate = 1;
		} else {
			offlineRate = 10;
			onlineRate = 1;
		}
	}

	public String readOutputConsole() {
		String output = String.valueOf(outputText_.getText());
		String[] outputs = output.split("\n");
		return outputs[outputs.length - 2];
	}

	// Called each time the user switches to the edit tab
	// to re load the contents of the file being edited.
	private void loadContentOfFileInEditTab(File f) {
		if (f == null) {
			// it can be that the file is not set if the user did not write
			// anything on the
			// editor and it didn't save a file.
			// DO NOTHING.
		} else {
			if (f.exists() && f.canRead()) {
				try {
					long len = f.length();
					byte buf[] = new byte[(int) len];
					FileInputStream fis = new FileInputStream(f);
					fis.read(buf);
					inputText_.setText(new String(buf, "UTF8"));
					addUsedFile(f);
				} catch (Exception e) {
					Log.e("AmbientTalk", "Could not read file " + f.getName(),
							e);
					Toast.makeText(this,
							"Could not load file (" + e.getMessage()
									+ ")\nStarting with an empty file ...",
							Toast.LENGTH_SHORT);
				}
			}
		}
	}

	// Called each time the user switches to the eval tab
	// to save the content of the file being edited.
	private void saveContentOfInputTextInEvalTab() {
		if (fileToEval_ == null) {
			// User did not save the contents of the file.
			// that happens when he clicks in "New File..." and
			// types content in the editor but doesn't save it via the Menu
			// view.
			// Current proposal: save it into a temporary file
			try {
				if (inputText_.getText().length() != 0) {
					File dest = File.createTempFile("tmp", ".at", new File(
							Constants._ENV_AT_BASE_,
							Constants._AT_TEMP_FILES_PATH));
					fileToEval_ = dest;
					saveContentOfEditFileToFile();
				}

			} catch (IOException e) {
				Log.e("AmbientTalk",
						"Could not save contents ot the editor to temp file ",
						e);
			}
		} else {
			// User is editing a previously saved file.
			// Current contents of the editor tab get saved to the file.
			// TODO: maybe ask the user if he wants to save changes.
			saveContentOfEditFileToFile();
		}
	}

	// Called when the user switches to the eval tab to save the contents of the
	// file displayed
	// or when the user click on the SaveFile item from the options menu.
	private void saveContentOfEditFileToFile() {
		try {
			String s = inputText_.getText().toString();
			byte buffer[] = s.getBytes();
			FileOutputStream fos = new FileOutputStream(fileToEval_);
			fos.write(buffer);
			fos.close();
			addUsedFile(fileToEval_);
		} catch (IOException e) {
			Log.e("AmbientTalk",
					"Could not save file " + fileToEval_.getName(), e);
			Toast.makeText(this, "Could not save file " + e.getMessage(),
					Toast.LENGTH_SHORT);
		}
	}

	@SuppressWarnings("unchecked")
	private void addUsedFile(File file) {
		Set<File> files = new HashSet<File>();
		try {
			ObjectInputStream ois = new ObjectInputStream(
					openFileInput("recently_used"));
			Set<File> current = (Set<File>) ois.readObject();
			files = current;
			ois.close();
		} catch (Exception e) {
			// File did not exist
		}
		files.add(file);
		try {
			ObjectOutputStream oos = new ObjectOutputStream(openFileOutput(
					"recently_used", MODE_PRIVATE));
			oos.writeObject(files);
		} catch (Exception e) {
			Log.e("AmbientTalk", "Could not write recently-used list", e);
		}
	}

	// The Options menu displayed in this activity
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.menu_atandroid, menu);
		return true;
	}

	// Called when user selects an item in the options menu.
	public boolean onOptionsItemSelected(MenuItem item) {
		final Activity me = this;
		Intent intent = null;
		switch (item.getItemId()) {
		case R.id.menu_Settings:
			intent = new Intent(this, Settings.class);
			startActivityForResult(intent, IntConstants._REQUEST_SETTINGS_);
			return true;
		case R.id.menu_Exit:
			setResult(IntConstants._RESULT_EXIT_);
			this.finish();
			return true;
		case R.id.menu_OpenFile:
			intent = new Intent(this, FileBrowser.class);
			// TODO: get the real AT_HOME here
			intent.putExtra(IntConstants._INTENT_START_PATH_,
					IATSettings.getDefaultIATOptions().AT_HOME_);
			startActivityForResult(intent, IntConstants._REQUEST_OPEN_FILE_);
			return true;
		case R.id.menu_SaveFile:
			if (fileToEval_ == null) {
				intent = new Intent(this, FileBrowser.class);
				intent.putExtra(IntConstants._INTENT_START_PATH_,
						IATSettings.getDefaultIATOptions().AT_HOME_);
				intent.putExtra(IntConstants._INTENT_NEW_FILE_, true);
				startActivityForResult(intent, IntConstants._REQUEST_SAVE_FILE_);
			} else {
				saveContentOfEditFileToFile();
			}
			return true;

		case R.id.menu_Reset:
			// Store the outputText string to an empty string
			// when you relaunch IAT.
			outputText_.setText("");
			saveOutputState("");
			IATManager.resetEnvironment(me);
			return true;
		case R.id.menu_RandomOffline:
			Intent simIntent = new Intent(this,
					RandomOfflineConfigActivity.class);
			simIntent.putExtra(
					getResources().getString(R.string.simulateCheckText),
					isSimEnabled);
			// 0 is random rates, 1 is fix rates
			simIntent.putExtra("randomRates", randomRates ? 0 : 1);
			simIntent.putExtra(getResources().getString(R.string.durationText),
					String.valueOf(simDuration));
			simIntent.putExtra(getResources()
					.getString(R.string.onlineRateText), String
					.valueOf(onlineRate));
			simIntent.putExtra(
					getResources().getString(R.string.offlineRateText),
					String.valueOf(offlineRate));
			simIntent.putExtra(getResources().getString(R.string.delayText),
					String.valueOf(delay));
			startActivityForResult(simIntent, SIM_REQUEST);
			return true;
		}
		return false;
	}

	/**
	 * ATAndroid expects results from 3 different activities launched when the
	 * user selects an item in the options menu.
	 */
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		switch (requestCode) {
		case (IntConstants._REQUEST_SETTINGS_): {
			if (resultCode == Activity.RESULT_OK) {
				Log.v("AmbientTalk", "Return OK of Settings activity");
				iatOptions_ = data
						.getParcelableExtra(IntConstants._INTENT_IAT_OPTIONS_);
				// TODO: copy back to iatOptions and restart if necessary
			}
			break;
		}
		case (IntConstants._REQUEST_OPEN_FILE_): {
			if (resultCode == Activity.RESULT_OK) {
				Log.v("AmbientTalk",
						"Return OK of File browser activity for an open file request");
				String filePath = data
						.getStringExtra(IntConstants._INTENT_PATH_);
				fileToEval_ = new File(filePath.trim());
				if (!fileToEval_.exists()) {
					System.out.println("File does not exist: "
							+ fileToEval_.getAbsolutePath());
				} else {
					try {
						String sourceCode = Evaluator
								.loadContentOfFile(fileToEval_);
						inputText_.setText(sourceCode);
					} catch (IOException e) {
						Toast.makeText(
								this,
								"Error reading file " + fileToEval_.getPath()
										+ ".Select again", Toast.LENGTH_SHORT)
								.show();
					}
				}

			}
			break;
		}
		case (IntConstants._REQUEST_SAVE_FILE_): {
			if (resultCode == Activity.RESULT_OK) {
				Log.v("AmbientTalk",
						"Return OK of File browser activity for a save file request");
				String filePath = data
						.getStringExtra(IntConstants._INTENT_PATH_);
				fileToEval_ = new File(filePath.trim());
				saveContentOfEditFileToFile();
			}

			break;
		}
		case SIM_REQUEST:
			if (data == null) {
				break;
			}
			if (aFrame1 == null) {
				Toast.makeText(this,
						"invalid setting, please initialize aFrame first.",
						Toast.LENGTH_LONG).show();
				break;
			}
			if (data.getBooleanExtra(
					getResources().getString(R.string.simulateCheckText), false)) {
				isSimEnabled = true;
				System.out.println("simulation enabled: true");
			} else {
				isSimEnabled = false;
				System.out.println("simulation enabled: false");
			}
			if (isSimEnabled) {
				if (data.getIntExtra("randomRates", 0) == 0) {
					randomRates = true;
					System.out.println("random rate set");
					try {
						offlineRate = Double.parseDouble(data
								.getStringExtra(getResources().getString(
										R.string.offlineRateText)));
						System.out.println("random offlineRate is: "
								+ offlineRate);
					} catch (Exception e) {
						offlineRate = 0.1;
						System.out
								.println("random offlineRate parsing error, using the default value: "
										+ offlineRate);
						Log.e(this.getClass().getName(), e.getMessage());
					}
					try {
						onlineRate = Double.parseDouble(data
								.getStringExtra(getResources().getString(
										R.string.onlineRateText)));
						System.out.println("random onlineRate is: "
								+ onlineRate);
					} catch (Exception e) {
						onlineRate = 1;
						System.out
								.println("random onlineRate parsing error, using the default value: "
										+ onlineRate);
						Log.e(this.getClass().getName(), e.getMessage());
					}
				} else if (data.getIntExtra("randomRates", 0) == 1) {
					randomRates = false;
					System.out.println("fix rate set");
					try {
						offlineRate = Double.parseDouble(data
								.getStringExtra(getResources().getString(
										R.string.offlineRateText)));
						System.out
								.println("fix offlineRate is: " + offlineRate);
					} catch (Exception e) {
						offlineRate = 10;
						System.out
								.println("fix offlineRate parsing error, using the default value: "
										+ offlineRate);
						Log.e(this.getClass().getName(), e.getMessage());
					}
					try {
						onlineRate = Double.parseDouble(data
								.getStringExtra(getResources().getString(
										R.string.onlineRateText)));
						System.out.println("fix onlineRate is: " + onlineRate);
					} catch (Exception e) {
						onlineRate = 1;
						System.out
								.println("fix onlineRate parsing error, using the default value: "
										+ onlineRate);
						Log.e(this.getClass().getName(), e.getMessage());
					}
				}
				try {
					simDuration = Double.parseDouble(data
							.getStringExtra(getResources().getString(
									R.string.durationText)));
					System.out
							.println("simulation duration is: " + simDuration);
				} catch (Exception e) {
					simDuration = 60000;
					System.out
							.println("simulation duration parsing error, using the default value: "
									+ simDuration);
					Log.e(this.getClass().getName(), e.getMessage());
				}
				if (randomRates) {
					IATManager.evaluate(
							" randomOfflineRateTest(true, "
									+ String.valueOf(simDuration) + ", "
									+ String.valueOf(offlineRate) + ", "
									+ String.valueOf(onlineRate) + ");", this);
				} else {
					IATManager.evaluate(
							" randomOfflineRateTest(false, "
									+ String.valueOf(simDuration) + ", "
									+ String.valueOf(offlineRate) + ", "
									+ String.valueOf(onlineRate) + ");", this);
				}
			}
			try {
				delay = Double.parseDouble(data.getStringExtra(getResources()
						.getString(R.string.delayText)));
				System.out.println("framework service delay is: " + delay);
			} catch (Exception e) {
				delay = 2000;
				System.out
						.println("framework service delay parsing error, using the default value: "
								+ delay);
				Log.e(this.getClass().getName(), e.getMessage());
			}
			IATManager.evaluate("residenceAgent.setFrameworkServiceDelay("
					+ String.valueOf((int)delay) + ");", this);
			getTabHost().setCurrentTab(1);
			break;
		}

	}

	// Store the output text in case of a screen rotation
	public Object onRetainNonConfigurationInstance() {
		return outputText_.getText();
	}

	// Store the output text when the activity moves to the foreground.
	public void onPause() {
		super.onPause();
		saveOutputState(outputText_.getText().toString());

	}

	private void saveOutputState(String textToStore) {
		SharedPreferences myPrefs = this.getSharedPreferences(
				IntConstants.ATANDROID, Context.MODE_PRIVATE);
		SharedPreferences.Editor prefsEditor = myPrefs.edit();
		prefsEditor
				.putString(IntConstants._PREFERENCE_OUTPUTTEXT_, textToStore);
		prefsEditor.commit();
	}

	// Restore the output text when the activity moves to the background.
	public void onResume() {
		super.onResume();
		if (IATManager._IAT_ == null) {
			finish();
		}
		SharedPreferences preferences = this.getSharedPreferences(
				IntConstants.ATANDROID, Context.MODE_PRIVATE);
		outputText_.setText(preferences.getString(
				IntConstants._PREFERENCE_OUTPUTTEXT_, ""));
	}

	// handler that receive the output/error messages from ambientTalk.
	private class ATViewUpdateHandler extends Handler {
		public void handleMessage(Message msg) {
			final SpannableString outputStr = new SpannableString(
					msg.obj.toString());
			switch (msg.what) {
			case IntConstants._ANDROID_INPUT_:
				outputStr.setSpan(new ForegroundColorSpan(Color.GREEN), 0,
						outputStr.length(), 0);
				break;
			case IntConstants._ANDROID_ERROR_:
				outputStr.setSpan(new ForegroundColorSpan(Color.RED), 0,
						outputStr.length(), 0);
				break;
			}
			outputText_.post(new Runnable() {
				public void run() {
					outputText_.append(outputStr);
					scrollView_.fullScroll(ScrollView.FOCUS_DOWN);
				}
			});

			super.handleMessage(msg);
		}
	}

	// used for debugging purposes
	private String errToString(Exception e) {
		ByteArrayOutputStream tmpMem = new ByteArrayOutputStream();
		PrintWriter tmp = new PrintWriter(tmpMem);
		e.printStackTrace(tmp);
		tmp.flush();
		return tmpMem.toString();
	}
}
