/**
 * AmbientTalk/2 Project
 * (c) Programming Technology Lab, 2006 - 2010
 * Authors: Software Languages Lab - Ambient Group
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
package edu.vub.at.android.interpreter;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import android.R.color;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.text.TextPaint;
import android.text.TextUtils;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.BaseAdapter;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import edu.vub.at.android.util.IntConstants;

public class FileBrowser extends Activity {
        
    protected static final int _CONTEXTMENU_SELECT_ITEM_ = 0;
    protected static final int _CONTEXTMENU_DELETE_ITEM_ = 1;
    protected static final int _CONTEXTMENU_EDIT_ITEM_   = 2;
    
    private File currentDirectory_ = new File("/");
    private ListView list_;
    private List<File> listEntries_ = new ArrayList<File>();
	private FileListAdapter flAdapter_;
	private boolean createNewFile_;

    /** Called when the activity is first created. */ 
    @Override 
    public void onCreate(Bundle savedInstanceState) {          
    	super.onCreate(savedInstanceState);
    	setContentView(R.layout.file_browser);
    	list_ = (ListView) findViewById(R.id.file_list);
    	
    	TextView tv = new TextView(this);
    	tv.setTextAppearance(this, android.R.style.TextAppearance_Large);
    	tv.setGravity(Gravity.CENTER);
    	tv.setText("Up one level ...");
    	tv.setTextSize(30);

    
		list_.addHeaderView(tv);
		
		createNewFile_ = getIntent().getBooleanExtra(IntConstants._INTENT_NEW_FILE_, false);
		if (createNewFile_) {
	    	TextView tv2 = new TextView(this);
	    	tv2.setTextAppearance(this, android.R.style.TextAppearance_Medium);
	    	tv2.setGravity(Gravity.CENTER);
	    	tv2.setText("Create new file");
			list_.addHeaderView(tv2);
		}
		
    	flAdapter_ = new FileListAdapter(this);
		list_.setAdapter(flAdapter_);
    	list_.setClickable(false);

    	list_.setOnItemClickListener(new FileClickListener());
    	
    	list_.setBackgroundColor(color.black);

    	browseToRoot();          
    } 
     
    /** 
     * This function browses to the root-directory of the file-system. 
     */ 
    private void browseToRoot() {
    	String start_path = getIntent().getStringExtra(IntConstants._INTENT_START_PATH_);
    	File root = Environment.getExternalStorageDirectory();
    	if (start_path != null) {
    		File f = new File(start_path);
    		if (f.exists())
    			root = f;
    	}
		browseTo(root);
    } 
     
    private void browseTo(final File aFile){ 
         if (aFile.isDirectory()){
             try {
				File current = aFile.getCanonicalFile();
				File[] files = current.listFiles();
				if (files == null)
					return;
				
				fill(files);
				currentDirectory_ = current;
			} catch (IOException e) { e.printStackTrace(); } 
         } else {
        	 currentDirectory_ = aFile.getParentFile();
         }
         
         String path = currentDirectory_.toString();
         int width = getWindowManager().getDefaultDisplay().getWidth();
         setTitle(TextUtils.ellipsize(path, new TextPaint(), width / 2, TextUtils.TruncateAt.MIDDLE));
         
         flAdapter_.notifyDataSetChanged();
    } 

    /*
     * Display list of files name in selected folder
     */
	private void fill(File[] files) {				
		listEntries_.clear();
		
		for (File file : files) {
			if (file.isFile() && ! (file.toString().endsWith(".at"))) {
				continue;
			}
			
			String fileName = file.getName();
			if (fileName.length() > 0 && !fileName.startsWith(".")) {	
				listEntries_.add(file);
			}
		}
		
		Collections.sort(listEntries_, new Comparator<File>() {
			public int compare(File f1, File f2) {
				boolean dir1 = f1.isDirectory();
				boolean dir2 = f2.isDirectory();
				
				if (dir1 == dir2)
					return f1.getName().compareToIgnoreCase(f2.getName());
				
				if (dir1)
					return -1;
				
				if (dir2)
					return 1;
				return 0;
			}	
		});
	}
    
    private final class FileClickListener implements OnItemClickListener {
		public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
			if (position == 0) {
				File parentFile = currentDirectory_.getParentFile();
				if (parentFile != null)
					browseTo(parentFile);
				return;
			}
			
			if (createNewFile_ && position == 1) {
				final EditText inputName = new EditText(FileBrowser.this);
				inputName.setHint("filename.at");
				
				new AlertDialog.Builder(FileBrowser.this)
					.setTitle("Filename?")
					.setView(inputName)
					.setPositiveButton("Create file", new OnClickListener() {
						public void onClick(DialogInterface dialog, int which) {
							String fileName = inputName.getText().toString().trim();
							if (!fileName.equals("")) {
								if (!fileName.endsWith(".at"))
									fileName = fileName + ".at";
								File newFile = new File(currentDirectory_, fileName);
								try {
									// TODO: if the file already exists, it is selected. Change this?
									newFile.createNewFile();
									Toast.makeText(FileBrowser.this, "Saved as '" + fileName + "'", Toast.LENGTH_SHORT).show();
									Intent ret = new Intent().putExtra(IntConstants._INTENT_PATH_, newFile.toString());
									setResult(RESULT_OK, ret);
									finish();
									return;
								} catch (IOException e) {
									// fall through
								}
							}
							Toast.makeText(FileBrowser.this, "Could not create file", Toast.LENGTH_SHORT).show();
						}
					}).show();
				return;
			}
			
			File clickedFile = (File) list_.getAdapter().getItem(position);
			
			if (clickedFile.isDirectory()) {
				browseTo(clickedFile);
			} else {
				final String filePath = clickedFile.toString();
				OnClickListener okButtonListener = new OnClickListener() { 
					public void onClick(DialogInterface arg0, int arg1) 
					{ 
						// Starts an intent to view the file that was clicked... 
						Log.v("Ambienttalk", "Load file....");
						Intent intent = new Intent();
						intent.putExtra(IntConstants._INTENT_PATH_, filePath);
						setResult(RESULT_OK, intent);
						finish();
					} 
				};
				
				Dialog d = new AlertDialog.Builder(FileBrowser.this) 
					.setTitle("Do you confirm your selection?") 
					.setMessage("Select "+ clickedFile.getName() + "?") 
					.setPositiveButton("OK", okButtonListener) 
					.setNegativeButton("Cancel", new OnClickListener(){ 
						public void onClick(DialogInterface d, int arg1) { d.cancel(); } 
					})
					.setCancelable(true)
					.create();
				
				d.setOwnerActivity(FileBrowser.this);
				d.show(); 

			}
		}
	}

	private class FileListAdapter extends BaseAdapter {
        public FileListAdapter(Context context) {
            mContext = context;
        }

        public int getCount() {
            return listEntries_.size();
        }

        public Object getItem(int position) {
            return listEntries_.get(position);
        }

        public long getItemId(int position) {
            return position;
        }

        public View getView(int position, View convertView, ViewGroup parent) {
        	View sv;
            if (convertView == null) {
                sv = LayoutInflater.from(mContext).inflate(R.layout.file_entry, null);
            } else {
                sv = convertView;
            }           

            TextView tv = (TextView) sv.findViewById(R.id.filename);
            tv.setText(listEntries_.get(position).getName());
            
            ImageView iv = (ImageView) sv.findViewById(R.id.icon);
            
            if (listEntries_.get(position).isDirectory()) {
            	iv.setImageResource(R.drawable.ic_menu_archive);
            } else {
            	iv.setImageResource(R.drawable.ic_new_file);
            }
            
            return sv;
        }

        private Context mContext;
    }
    
}