package edu.vub.at.android.interpreter;

import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Set;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.TextView;
import edu.vub.at.android.util.ATLibInstaller;
import edu.vub.at.android.util.Constants;
import edu.vub.at.android.util.IATSettings;
import edu.vub.at.android.util.IntConstants;

public class Greeter extends Activity {
	private RecentlyUsedAdapter recentlyUsedAdapter_;
	private ListView listView_;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.greeter);
		
		final ArrayList<File> items = getUsedFiles();
		
		listView_ = (ListView) findViewById(R.id.recently_used_list);
		
		View v = getLayoutInflater().inflate(android.R.layout.simple_list_item_1, null);
		((TextView) v.findViewById(android.R.id.text1)).setText("No files found.");
		listView_.setEmptyView(v);
		((ViewGroup) listView_.getParent()).addView(v);

		recentlyUsedAdapter_ = new RecentlyUsedAdapter(this, android.R.layout.simple_list_item_1, items);
		listView_.setAdapter(recentlyUsedAdapter_);
		
		listView_.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				File f = items.get(position);
				startEvalPanel(f.toString());
			}
		});
		
		Button newButton = (Button) findViewById(R.id.new_file_button);
		newButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				startEvalPanel(null); // No file.
			}
		});
		
		Button openButton = (Button) findViewById(R.id.open_file_button);
		openButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				Intent fileBrowserIntent = new Intent(Greeter.this, FileBrowser.class);
				// TODO: get the real AT_HOME here
				fileBrowserIntent.putExtra(IntConstants._INTENT_START_PATH_, IATSettings.getDefaultIATOptions().AT_HOME_);
				startActivityForResult(fileBrowserIntent, IntConstants._REQUEST_BROWSE_);
			}
		});
		
		// only start iat the first time the activity launches.
		if (   getLastNonConfigurationInstance() == null && savedInstanceState == null
			|| !IATManager.canStart()) {
			Intent i = new Intent(this, ATLibInstaller.class);
			startActivityForResult(i, IntConstants._REQUEST_ATLIB_INSTALL_);
		}
		
		// show xmas logo between xmas and ny
		Date now = new Date();
		Date xmas = new Date(now.getYear(), 11, 25);
		Date nyr = new Date(now.getYear() + 1, 0, 1);
		if (now.after(xmas) && now.before(nyr)) {
			ImageView logo = (ImageView) findViewById(R.id.ATLogo);
			logo.setImageResource(R.drawable.atlogo_xmas);
		}
	}
	
	@SuppressWarnings("unchecked")
	private ArrayList<File> getUsedFiles() {
		ArrayList<File> ret = new ArrayList<File>();
		try {
			ObjectInputStream ois = new ObjectInputStream(openFileInput("recently_used"));
			Set<File> stored = (Set<File>) ois.readObject();
			for (File f: stored)
				if (f.canRead())
					ret.add(f);

			Collections.sort(ret, new Comparator<File>() {
				public int compare(File object1, File object2) {
					return (int) (object2.lastModified() - object1.lastModified());
				}
			});
		} catch (IOException e) {
			//e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}

		return ret;
	}
	
	@Override
	public Object onRetainNonConfigurationInstance() {
		return this;
	}

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		recentlyUsedAdapter_.changeFiles( getUsedFiles() );
		
    	switch(requestCode) { 
    	case (IntConstants._REQUEST_ATLIB_INSTALL_) : { 
    		if (resultCode == Constants._RESULT_FAIL_) {
				finish();
			} else {
				startATThread();
			}
    		break; 
    	}
    	case (IntConstants._REQUEST_BROWSE_) : { 
    		if (resultCode == RESULT_OK && data.hasExtra(IntConstants._INTENT_PATH_)) {
				String path = data.getStringExtra(IntConstants._INTENT_PATH_);
				startEvalPanel(path);
			} 
    		break; 
    	}
    	case (IntConstants._REQUEST_ATANDROID_) : {
    		if (resultCode == IntConstants._RESULT_EXIT_) {
    			// Store the outputText string to an empty string  
    			// so that next run the output is clean and exit
    			saveOutputState();
    			System.exit(0);
    		}
    		break;
    	}
    	}
	}
	
	//Used to save the output text preference when the ATAndroid finishes or before launching a new IAT.
	private void saveOutputState() {
		SharedPreferences myPrefs =  this.getSharedPreferences(IntConstants.ATANDROID, Context.MODE_PRIVATE);
		SharedPreferences.Editor prefsEditor = myPrefs.edit();
		prefsEditor.putString(IntConstants._PREFERENCE_OUTPUTTEXT_, "");
		prefsEditor.commit();
	}
	
	private void startEvalPanel(String path) {
		Intent i = new Intent(this, ATAndroid.class);
		if (path != null)
			i.putExtra(IntConstants._INTENT_LOAD_FILE_, path);
		startActivityForResult(i, IntConstants._REQUEST_ATANDROID_);
	}
	 
	private void startATThread() {
		final Activity me = this;
		saveOutputState();
		IATManager.pokeIAT(me);
	}

    public boolean onCreateOptionsMenu(Menu menu) {
    	MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_greeter, menu);
        return true;
    }

    public boolean onOptionsItemSelected(MenuItem item){    
    	  switch (item.getItemId()) {
          case R.id.menu_Settings:
        	  Intent intent = new Intent(this, Settings.class);
        	  startActivityForResult(intent, IntConstants._REQUEST_SETTINGS_);
              return true;
          case R.id.menu_Exit:
        	  saveOutputState();
        	  System.exit(0);
        	  return true;
    	  }	 
          return false;   
    }
  
	public class RecentlyUsedAdapter extends ArrayAdapter<File> implements ListAdapter {

		private ArrayList<File> files;

		public RecentlyUsedAdapter(Context context, int textViewResourceId, ArrayList<File> files) {
			super(context, textViewResourceId, files);
			this.files = files;
		}
		
		public void changeFiles(ArrayList<File> files) {
			this.files = files;
			clear();
			for (File file : files) {
				this.add(file);
			}
			this.notifyDataSetChanged();
		}
		
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View v = convertView;
			if (v == null)
				v = getLayoutInflater().inflate(android.R.layout.simple_expandable_list_item_2, null);
			TextView filename = (TextView) v.findViewById(android.R.id.text1);
			filename.setText(files.get(position).getName());
			TextView date = (TextView) v.findViewById(android.R.id.text2);
			date.setText(new Date(files.get(position).lastModified()).toLocaleString());
			return v;
		}
	}
}
