package edu.vub.at.android.interpreter;

import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.util.Log;
import android.widget.Toast;
import edu.vub.at.IAT;
import edu.vub.at.android.util.Constants;
import edu.vub.at.android.util.IATAndroid;
import edu.vub.at.android.util.OutputAT;
import edu.vub.at.android.util.R;

public class IATManager {
	
	public static IAT _IAT_;
	private static boolean startInProgress_ = false;
	
	private IATManager() {}
	
	public static void pokeIAT(final Activity act) {
		final Runnable run = new Runnable() {
			public void run() {
				try {
					if (!IATManager.startInProgress_) {
						startInProgress_ = true;
						// Store the outputText string to an empty string  
						// when you relaunch IAT.
						//saveOutputState();
						System.setOut(OutputAT.out);
						System.setErr(OutputAT.err);
						// detach the output in case you are here 
						// because the ATAndroid was killed, and Greeter 
						// has been asked to relauch iat.
						// Detaching makes the output buffer messages
						// that the interpreter may send while 
						// ATAndroid view is not created and attaches
						// itself to the output.
						OutputAT.out.detach();
						OutputAT.err.detach();
						_IAT_ = IATAndroid.create(act);
						startInProgress_ = false;
					}
				} catch (Exception e) {
					Log.e("AmbientTalk","Could not start IAT", e);
					startInProgress_ = false;
				}
			}
		};
		new Thread(run).start();
	}
	
	public static void evaluate(final String input, final Activity act) {
		if (input.length() <= 0)
			return;
		final ProgressDialog pd = ProgressDialog.show(act, "", "Evaluating", true);
		Thread t = new Thread() {
			public void run() {
				try {
					while (_IAT_ == null) {
						try {
							Thread.sleep(500);
						} catch (final Exception e) {
							// do nothing
						}
					}
					_IAT_.evalAndPrint(input, System.out);
					pd.cancel();
				} catch (final Exception e) {
					pd.cancel();
					act.runOnUiThread(new Runnable() { public void run() {
						IATManager.showReportErrorDialog(e.getMessage(), e, act);
					}});
					Log.v("AmbientTalk", errToString(e), e);
				}
			}
		};
		t.start();
	}
	
	public static void resetEnvironment(final Activity act){
		final ProgressDialog pd = ProgressDialog.show(act, "", "Reseting Evaluation Environment", true);
		Thread t = new Thread() {
			public void run() {
				try {
					if( _IAT_ != null) {
						int res = _IAT_.softReset();
						if (res != 0 ) {
							act.runOnUiThread(new Runnable() { public void run() {
								Toast.makeText(act, "Reset the evaluation environment failed, try again.", Toast.LENGTH_SHORT).show();}
							});
						}
					}
					pd.cancel();
				} catch (final Exception e) {
					// TODO: it shouldn't throw an exception anymore.
					pd.cancel();
					act.runOnUiThread(new Runnable() { public void run() {
						Toast.makeText(act, "Reset the evaluation environment failed:\n"+ e.getMessage(), Toast.LENGTH_SHORT).show();}});
					Log.v("AmbientTalk", errToString(e), e);
				}
			}
		};
		t.start();
	}
	
	private static String errToString(Exception e) {
		ByteArrayOutputStream tmpMem = new ByteArrayOutputStream();
		PrintWriter tmp = new PrintWriter(tmpMem);
		e.printStackTrace(tmp);
		tmp.flush();
		return tmpMem.toString();
	}
	
    private static void showReportErrorDialog(final String message, final Throwable t, final Activity act) {
		AlertDialog.Builder builder = new AlertDialog.Builder(act);
		AlertDialog sendErrorReportDialog = builder.create();
		sendErrorReportDialog.setTitle("AmbientTalk encountered an error");
		sendErrorReportDialog.setMessage("Do you want to report this error?\n" + t.getMessage());
		sendErrorReportDialog.setIcon(R.drawable.ic_dialog_alert);
		sendErrorReportDialog.setButton(AlertDialog.BUTTON_POSITIVE, "Yes", new AlertDialog.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				final Intent emailIntent = new Intent(Intent.ACTION_SEND);
                emailIntent.setType("plain/text");
                emailIntent.putExtra(Intent.EXTRA_EMAIL, new String[]{ Constants._AT_EMAIL_ADDRESS_ });
                emailIntent.putExtra(Intent.EXTRA_SUBJECT, "[AT crash report] "+ message);
                String traceString = "";
                StackTraceElement[] trace = t.getStackTrace();
                for (int i = 0; i < trace.length; i++) {
                	traceString = traceString + (trace[i]).toString() + "\n";
                }
                emailIntent.putExtra(Intent.EXTRA_TEXT, traceString);
                act.startActivity(Intent.createChooser(emailIntent, "Send mail..."));
                dialog.dismiss();
                System.out.println(message);
        		System.exit(1);
			}
		});
		sendErrorReportDialog.setButton(AlertDialog.BUTTON_NEGATIVE, "No", new AlertDialog.OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
				System.out.println(message);
				System.exit(1);
			}
		});
		sendErrorReportDialog.show();
	}
	
	public static boolean canStart() {
		return (startInProgress_ || _IAT_ != null);
	}

}
