package edu.vub.at.android.interpreter;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.Spinner;

public class RandomOfflineConfigActivity extends Activity {
	private static CheckBox simEnabledCheckBox;
	private EditText simDuration;
	private EditText offlineRate;
	private EditText onlineRate;
	private EditText delay;

	private static final String[] randomRate = { "random rates", "fix rates" };
	private Spinner spinner;
	private ArrayAdapter<String> adapter;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_rancom_offline_config);

		spinner = (Spinner) findViewById(R.id.randomRateSpinner);
		adapter = new ArrayAdapter<String>(this,
				android.R.layout.simple_spinner_item, randomRate);
		adapter.setDropDownViewResource(android.R.layout.simple_spinner_item);
		spinner.setAdapter(adapter);
		spinner.setOnItemSelectedListener(new SpinnerSelectedListener());
		spinner.setVisibility(View.VISIBLE);

		simDuration = (EditText) findViewById(R.id.duration);
		offlineRate = (EditText) findViewById(R.id.offlineRate);
		onlineRate = (EditText) findViewById(R.id.onlineRate);
		delay = (EditText) findViewById(R.id.delay);
		simEnabledCheckBox = (CheckBox) findViewById(R.id.checkSimulation);
		Intent intent = getIntent();
		Double delayValue;
		try {
			delayValue = Double.parseDouble(intent
					.getStringExtra(getResources()
							.getString(R.string.delayText)));
			System.out.println("framework service delay is: " + delayValue);
		} catch (Exception e) {
			delayValue = 2000.0;
			System.out
					.println("framework service delay parsing error, using the default value: "
							+ delayValue);
			Log.e(this.getClass().getName(), e.getMessage());
		}
		delay.setText(String.valueOf(delayValue));
		if (!intent.getBooleanExtra(
				getResources().getString(R.string.simulateCheckText), false)) {
			simEnabledCheckBox.setChecked(false);
			simDuration.setEnabled(false);
			offlineRate.setEnabled(false);
			onlineRate.setEnabled(false);
			spinner.setEnabled(false);
		} else {
			simEnabledCheckBox.setChecked(true);
			simDuration.setEnabled(true);
			offlineRate.setEnabled(true);
			onlineRate.setEnabled(true);
			spinner.setEnabled(true);

			if (intent.getIntExtra("randomRates", 0) == 0) {
				spinner.setSelection(0);
			} else if (intent.getIntExtra("randomRates", 0) == 1) {
				spinner.setSelection(1);
			}
			offlineRate.setText(intent.getStringExtra(getResources().getString(
					R.string.offlineRateText)));
			onlineRate.setText(intent.getStringExtra(getResources().getString(
					R.string.onlineRateText)));
			simDuration.setText(intent.getStringExtra(getResources().getString(
					R.string.durationText)));
		}

		simEnabledCheckBox
				.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
					public void onCheckedChanged(CompoundButton buttonView,
							boolean isChecked) {
						if (isChecked) {
							simDuration.setEnabled(true);
							offlineRate.setEnabled(true);
							onlineRate.setEnabled(true);
							spinner.setEnabled(true);
						} else {
							simDuration.setEnabled(false);
							offlineRate.setEnabled(false);
							onlineRate.setEnabled(false);
							spinner.setEnabled(false);
						}
					}
				});

	}

	class SpinnerSelectedListener implements OnItemSelectedListener {
		public void onItemSelected(AdapterView<?> arg0, View arg1, int arg2,
				long arg3) {
			if (arg2 == 0) {
			} else if (arg2 == 1) {
			}
		}

		public void onNothingSelected(AdapterView<?> arg0) {

		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.activity_rancom_offline_config, menu);
		return true;
	}

	public void onOKClicked(View view) {
		Intent intent = getIntent();
		intent.putExtra(getResources().getString(R.string.simulateCheckText),
				simEnabledCheckBox.isChecked());
		// 0 is random rates, 1 is fix rates
		intent.putExtra("randomRates", spinner.getSelectedItemPosition());
		intent.putExtra(getResources().getString(R.string.durationText),
				simDuration.getText().toString());
		intent.putExtra(getResources().getString(R.string.onlineRateText),
				onlineRate.getText().toString());
		intent.putExtra(getResources().getString(R.string.offlineRateText),
				offlineRate.getText().toString());
		intent.putExtra(getResources().getString(R.string.delayText), delay
				.getText().toString());
		this.setResult(RESULT_OK, intent);
		finish();
	}

	public void onCancelClicked(View view) {
		finish();
	}
}
