/**
 * AmbientTalk/2 Project
 * (c) Programming Technology Lab, 2006 - 2010
 * Authors: Software Languages Lab - Ambient Group
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
package edu.vub.at.android.interpreter;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.DialogInterface.OnClickListener;
import android.os.Bundle;
import android.view.Menu;
import android.view.View;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import edu.vub.at.android.util.Constants;
import edu.vub.at.android.util.IATSettings;
import edu.vub.at.android.util.IntConstants;
import edu.vub.at.android.util.IATSettings.IATOptions;

/*
 * This activity allows the user to change the settings. 
 * The data model of a single entry in the list is a HashMap from type String to Object 
 *
 * _FROM_ 			->  Label value
 * _TO_ 			->  Text value
 * _TYPE_ 			-> 	PATH, IP, NETWORK_NAME
 * _REQUEST_CODE_ 	->	The ID used in order to see which activity returned 
 * 
 */
public class Settings extends ListActivity {
	
	private static String _FROM_ = "name";
	private static String _TO_ = "value";
	private static String _TYPE_ = "type";
	private static String _REQUEST_CODE_ = "resultCode";
	
	private static enum Type { PATH, IP, NETWORK_NAME, CLEAR_RECENT, CLEAR_TEMPORARY };
	private IATOptions iatOptions_;
	private List<HashMap<String,Object>> listData_;
	
	/* 
	 * When the list activity is created we populate the list
	 * with the various options. 
	 */
	@SuppressWarnings("serial")
	public void onCreate(Bundle savedInstanceState) {    
		super.onCreate(savedInstanceState);  
		//Retrieve the IAT options, these are from persistent storage.
		iatOptions_ = IATSettings.getIATOptions(this);
		listData_ = new ArrayList<HashMap<String,Object>>();
		
		//Create the entry for choosing the AT_HOME path 
		listData_.add(new HashMap<String,Object>() {
		{ 	put(Settings._FROM_, "Path to atlib");  
			put(Settings._TO_, iatOptions_.AT_HOME_);  
			put(Settings._TYPE_, Settings.Type.PATH); 
			put(Settings._REQUEST_CODE_, IntConstants._REQUEST_SELECT_AT_HOME_); }
		});
		
		//Create the entry for choosing the the init file. 
		listData_.add(new HashMap<String,Object>() {
		{	put(Settings._FROM_, "AT init file");
			put(Settings._TO_, iatOptions_.AT_INIT_);
			put(Settings._TYPE_, Settings.Type.PATH);
			put(Settings._REQUEST_CODE_, IntConstants._REQUEST_SELECT_INIT_PATH_); }
		});
		
		//Create the entry for choosing the log file location
		listData_.add(new HashMap<String,Object>() {
		{ 	put(Settings._FROM_, "Path to logfile"); 
			put(Settings._TO_, iatOptions_.logFilePath_); 
			put(Settings._TYPE_, Settings.Type.PATH); 
			put(Settings._REQUEST_CODE_, IntConstants._REQUEST_SELECT_LOG_FILE_); }
		});
		
		//Create the entry for choosing the network name
		listData_.add(new HashMap<String,Object>() {
		{ 	put(Settings._FROM_, "Network name");
			put(Settings._TO_, iatOptions_.networkName_); 
			put(Settings._TYPE_, Settings.Type.NETWORK_NAME);}
		});
		
		//Create the entry for choosing the ip adress
		listData_.add(new HashMap<String,Object>() {
		{ 	put(Settings._FROM_, "IP address");
			put(Settings._TO_, iatOptions_.ipAddress_);
			put(Settings._TYPE_, Settings.Type.IP );}
		});
		// Create the entry for clearing the recently used files
		listData_.add(new HashMap<String,Object>() {
		{	put(Settings._FROM_, "Clear recently used files list");
			put(Settings._TYPE_, Settings.Type.CLEAR_RECENT );
			put(Settings._REQUEST_CODE_, IntConstants._REQUEST_CLEAR_RECENT_FILES_); }
		});
		
		// Create the entry for clearing the temporary files created by editor.
		listData_.add(new HashMap<String,Object>() {
		{	put(Settings._FROM_, "Clear temporary files in cache");
			put(Settings._TYPE_, Settings.Type.CLEAR_TEMPORARY );
			put(Settings._REQUEST_CODE_, IntConstants._REQUEST_CLEAR_TEMPORARY_FILES_); }
		});

		setListAdapter(new SimpleAdapter(this, listData_,
				android.R.layout.simple_list_item_2,
				new String[] { Settings._FROM_, Settings._TO_ },
				new int[] { android.R.id.text1, android.R.id.text2 }));
	}

	public boolean onCreateOptionsMenu(Menu menu) {
  	  	Intent intent = new Intent(this, TweetsSearchActivity.class);
		startActivity(intent);
		return true;
	}

	protected void onListItemClick(ListView l, View v, final int position, long id) {
		super.onListItemClick(l, v, position, id);
		Type type = (Type) listData_.get(position).get(Settings._TYPE_);
		switch(type) {
		case PATH:
			int code = (Integer) listData_.get(position).get(Settings._REQUEST_CODE_);
			Intent intent = new Intent(this, FileBrowser.class);
			// Set the currently selected path as default, if possible
			String current = (String) listData_.get(position).get(Settings._TO_);
			if (new File(current).exists()) {
				intent.putExtra(IntConstants._INTENT_START_PATH_, current);
			}
			startActivityForResult(intent, code);
			break;
			
		case IP:
			Set<String> addresses = IATSettings.eligibleIpAddresses();
			String title = (String) listData_.get(position).get(Settings._FROM_);
			final String addressesArr[] = addresses.toArray(new String[] {});
			new AlertDialog.Builder(this)
			.setItems(addressesArr, new OnClickListener() {
				public void onClick(DialogInterface dialog, int which) {
					iatOptions_.ipAddress_ = addressesArr[which];	
					listData_.get(position).put(Settings._TO_, iatOptions_.ipAddress_);			
					onContentChanged();
				}}).setTitle(title + "?").create().show();
			break;
			
		case CLEAR_TEMPORARY:
			AlertDialog dialog = new AlertDialog.Builder(this).create();
			dialog.setButton(AlertDialog.BUTTON_POSITIVE, "OK", new AlertDialog.OnClickListener() {
				public void onClick(DialogInterface dialog, int which) {
	                dialog.dismiss();
				}
			});
			File cacheDir = new File (Constants._ENV_AT_BASE_,Constants._AT_TEMP_FILES_PATH);
			File [] cacheDirFile = cacheDir.listFiles();
			boolean result = true;
			for (int i = 0; i < cacheDirFile.length; i++) {
				File file = cacheDirFile[i];
				result = result && file.delete();
				deleteFile(file.getName());
			}
			if (result) {
				dialog.setMessage("Temporary files cleared.");
				dialog.setIcon(R.drawable.ic_dialog_info);
			} else {
				dialog.setMessage("Could not clear all temporary files in cache");
				dialog.setIcon(R.drawable.ic_dialog_alert);
			}
			dialog.show();
			break;
			
		case CLEAR_RECENT:	
			AlertDialog dialogR = new AlertDialog.Builder(this).create();
			dialogR.setButton(AlertDialog.BUTTON_POSITIVE, "OK", new AlertDialog.OnClickListener() {
				public void onClick(DialogInterface dialog, int which) {
	                dialog.dismiss();
				}
			});
			if (deleteFile("recently_used")) {
				dialogR.setMessage("Recently used files list cleared.");
				dialogR.setIcon(R.drawable.ic_dialog_info);
			} else {
				dialogR.setMessage("Could not clear recently used files list!");
				dialogR.setIcon(R.drawable.ic_dialog_alert);
			}
			dialogR.show();
			break;
			
		case NETWORK_NAME:
			AlertDialog.Builder alert = new AlertDialog.Builder(this);  
			  
			alert.setTitle((String) listData_.get(position).get(Settings._FROM_));  
			alert.setMessage("");  
			  
			// Set an EditText view to get user input   
			final EditText input = new EditText(this);  
			input.setText((String)listData_.get(position).get(Settings._TO_));
			alert.setView(input);
			
			  
			alert.setPositiveButton("Ok", new DialogInterface.OnClickListener() {  
			public void onClick(DialogInterface dialog, int whichButton) {  
				String value = input.getText().toString();
				listData_.get(position).put(Settings._TO_, value);
				iatOptions_.networkName_ = value;
				onContentChanged();
				new SettingsDialog(Settings.this).show();
			  }  
			});  
			  
			alert.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {  
			  public void onClick(DialogInterface dialog, int whichButton) {  
			    // Canceled.  
			  }  
			});  
			  
			alert.show(); 
		}
	}

	/*
	 * Handle the result from the activities spawned from clicking on the list items.
	 * @see android.app.Activity#onActivityResult(int, int, android.content.Intent)
	 */
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (resultCode != RESULT_OK)
			return;

		String filePath = data.getStringExtra(IntConstants._INTENT_PATH_);
		if (filePath == null || filePath == "")
			return;

		File filePathFile = new File(filePath);

		switch (requestCode) {
		case IntConstants._REQUEST_SELECT_AT_HOME_:
			if (filePathFile.isFile())
				filePath = filePathFile.getParentFile().toString();

			listData_.get(0).put(Settings._TO_, filePath);
			iatOptions_.AT_HOME_ = filePath;
			break;

		case IntConstants._REQUEST_SELECT_INIT_PATH_:
			listData_.get(1).put(Settings._TO_, filePath);
			iatOptions_.AT_INIT_ = filePath;
			break;

		case IntConstants._REQUEST_SELECT_LOG_FILE_:
			// Check filePath is a path of file or directory
			File logFile = new File(filePath);
			String newPath = null;
			if (logFile.exists() && logFile.isFile()) {
				newPath = filePath;
			} else if (logFile.exists()) {
				newPath = filePath + "/log.txt";
			} else {
				return; // buh?
			}
			iatOptions_.logFilePath_ = newPath;
			listData_.get(2).put(Settings._TO_, newPath);
			break;
		}
		onContentChanged();
		new SettingsDialog(this).show();			
	}
	
	
	

	protected void onDestroy() {
		super.onDestroy();
		
		SharedPreferences myPrefs = this.getSharedPreferences(Constants.IAT_SETTINGS_FILE, Context.MODE_PRIVATE);
		SharedPreferences.Editor prefsEditor = myPrefs.edit();
		iatOptions_.writeToPreferences(prefsEditor);
		prefsEditor.commit();

		Intent i = new Intent();
		i.putExtra(IntConstants._INTENT_IAT_OPTIONS_, iatOptions_);
		setResult(RESULT_OK, i);
		
	}
	
	

	// Return the (possibly unchanged) options
	protected void onPause() {
		super.onPause();

		//TODO move to Settings
		SharedPreferences myPrefs =  this.getSharedPreferences(Constants.IAT_SETTINGS_FILE, Context.MODE_PRIVATE);
		SharedPreferences.Editor prefsEditor = myPrefs.edit();
		iatOptions_.writeToPreferences(prefsEditor);
		prefsEditor.commit();

		Intent i = new Intent();
		i.putExtra(IntConstants._INTENT_IAT_OPTIONS_, iatOptions_);
		setResult(RESULT_OK, i);
	}
}
