package mobileAgent;

import java.util.Collection;

import edu.vub.at.IAT;

public class GeneralMobileAgent {
	int pDelay = 500;
	private IAT vm;
	String id = "";

	public GeneralMobileAgent(IAT iat) {
		this.vm = iat;
	}

	public String setId(String id) {
		this.id = id;
		return id;
	}

	public String getId() {
		return id;
	}

	public void setATVM(IAT vm) {
		this.vm = vm;
	}

	/**
	 * This method generates a application specific mobile agent from the
	 * general template.
	 * 
	 * @param key
	 *            the identifier of the mobile agent. The key identifies when
	 *            mobile agent transite to one node, what kind of framework
	 *            services or application specific services to invoke.
	 * @param type
	 *            type is used to denote the type of the mobile agent. It has
	 *            three optional values, if type equals to "1", the mobile agent
	 *            will go to invoke framework service identified by the key
	 *            parameter. If type equals to "2", the mobile agent will go to
	 *            invoke applicaiton specific service identified by the key
	 *            parameter. If type equels to "3", the mobile agent will go to
	 *            read the meta-data of the data file, for example, the length
	 *            of the data file and the creation time of the meta-data file.
	 *            If type equals "4", the mobile agent will invoke firstly one
	 *            application specific service identified by the key parameter.
	 *            This service will return a string value, which is a name of
	 *            another service, mobile agent will then invoke the second
	 *            service again. In this way, the second service to be invoked
	 *            can be decided dynamically.
	 */
	public void getApplicationSpecificMobileAgent(String key, String type) {
		vm.evalAndPrint("ma" + id + ".getApplicationSpecificMobileAgent(\""
				+ key + "\", \"" + type + "\");", System.out);
	}

	/**
	 * set the owner id of the mobile agent. Once owner id is set, when mobile
	 * agent wants to go back to owner, it can decide which node to go back.
	 * 
	 * @param name
	 *            the id of the mobile agent owner.
	 */
	public void setOwner(String name) {
		vm.evalAndPrint("ma" + id + ".setOwner(\"" + name + "\");", System.out);
	}

	/**
	 * set the name of the application of the mobile agent. Once the application
	 * which creates the mobile agent is set to mobile agent. When mobile agent
	 * come back to the owner node, it can know to which application to submit
	 * the data it takes back from the ad-hoc network.
	 * 
	 * @param appName
	 *            the name of the application which creates the mobile agent.
	 */
	public void setAppName(String appName) {
		vm.evalAndPrint("ma" + id + ".setAppName(\"" + appName + "\");",
				System.out);
	}

	public void setDelay(int pDelay) {
		this.pDelay = pDelay;
		vm.evalAndPrint("ma" + id + ".setDelay(" + pDelay + ");", System.out);
	}

	public int getDelay() {
		return pDelay;
	}

	/**
	 * The mobile agent can transition through the ad-hoc network in three
	 * modes: through all the nodes in the network, transite only one step to
	 * other nodes or transition according to a subset of the node set of the
	 * ad-hoc network.
	 * 
	 * @param mode
	 *            transiton mode, can only be one of the three values: allNodes,
	 *            oneStep or subSet. allNodes mode make mobile agent go through
	 *            all the nodes of the ad-hoc network. subSet mode makes the
	 *            mobile agent transits through the ad-hot network according to
	 *            a sub set. oneStep mode makes the mobile agent to go to only
	 *            one node and stop there without coming back.
	 */
	public void setTransitionMode(String mode) {
		vm.evalAndPrint("ma" + id + ".setTransitionMode(\"" + mode + "\");",
				System.out);
	}// optional values: allNodes,
		// oneStep, subSet.

	/**
	 * When the transition is set to "subSet", mobile agent can transite only
	 * through a sub set of all the nodes of the ad-hoc network. The sub set is
	 * set using this method. When the transition is set to "oneStep", mobile
	 * agent can transite only one step, the target of this one step is set
	 * using this method.
	 * 
	 * @param set
	 *            the sub set of all the nodes in the ad-hoc network.
	 */
	public void setTransitionSet(Collection<String> set) {
		vm.evalAndPrint("def trSubSet" + this.getId()
				+ " := jlobby.java.util.ArrayList.new();", System.out);
	}

	/**
	 * execute service in a node which mobile agent transites to.
	 */
	public void execute() {
		vm.evalAndPrint("ma" + id + ".execute();", System.out);
	}

	/**
	 * make the mobileagent transition to the next node of the ad-hoc network.
	 * 
	 * @param clean
	 *            boolean value to show whether to clean data it left at the
	 *            residence agent. True value mean to clean data.
	 * @param takenAway
	 *            boolean value to shown whether take the data away with the
	 *            mobile agent. True value means to take the data away with
	 *            mobile agent.
	 */
	public void transitToNextNode(boolean clean, boolean takenAway) {
		vm.evalAndPrint("ma" + id + ".transitToNextNode(true, true);",
				System.out);
	}

}
